@@ -8,10 +8,13 @@ use std::error::Error;
 use log::info;
 use serde::{Deserialize, Serialize};
 
+pub(crate) const DEFAULT_GITHUB_API_URL: &'static str = "https://api.github.com";
+
 #[derive(Debug)]
 pub enum GithubError {
     JsonWebTokenCreation(jsonwebtoken::errors::Error),
     HttpError(reqwest::Error),
+    CommentTooLarge(String),
 }
 
 impl Error for GithubError {
@@ -19,6 +22,7 @@ impl Error for GithubError {
         match self {
             GithubError::JsonWebTokenCreation(err) => Some(err),
             GithubError::HttpError(err) => Some(err),
+            GithubError::CommentTooLarge(_) => None,
         }
     }
 }
