@@ -1,20 +1,11 @@
 use lazy_static::lazy_static;
 use std::collections::HashSet;
 
-use squawk_syntax::ast;
 use squawk_syntax::ast::AstNode;
-use squawk_syntax::{Parse, SourceFile};
+use squawk_syntax::{Parse, SourceFile, SyntaxKind};
+use squawk_syntax::{ast, identifier::Identifier};
 
-use crate::identifier::Identifier;
-use crate::{Linter, Rule, Violation};
-
-fn is_const_expr(expr: &ast::Expr) -> bool {
-    match expr {
-        ast::Expr::Literal(_) => true,
-        ast::Expr::CastExpr(cast) => matches!(cast.expr(), Some(ast::Expr::Literal(_))),
-        _ => false,
-    }
-}
+use crate::{Linter, Rule, Version, Violation};
 
 lazy_static! {
     static ref NON_VOLATILE_FUNCS: HashSet<Identifier> = {
@@ -27,8 +18,18 @@ lazy_static! {
     };
 }
 
-fn is_non_volatile(expr: &ast::Expr) -> bool {
+fn is_non_volatile_or_const(expr: &ast::Expr) -> bool {
     match expr {
+        ast::Expr::Literal(_) => true,
+        ast::Expr::ArrayExpr(_) => true,
+        ast::Expr::BinExpr(bin_expr) => {
+            if let Some(lhs) = bin_expr.lhs() {
+                if let Some(rhs) = bin_expr.rhs() {
+                    return is_non_volatile_or_const(&lhs) && is_non_volatile_or_const(&rhs);
+                }
+            }
+            false
+        }
         ast::Expr::CallExpr(call_expr) => {
             if let Some(arglist) = call_expr.arg_list() {
                 let no_args = arglist.args().count() == 0;
@@ -46,6 +47,24 @@ fn is_non_volatile(expr: &ast::Expr) -> bool {
                 false
             }
         }
+        // array[]::t[] is non-volatile. We don't check for a plain array expr
+        // since postgres will reject it as a default unless it's cast to a type.
+        ast::Expr::CastExpr(cast_expr) => {
+            if let Some(inner_expr) = cast_expr.expr() {
+                is_non_volatile_or_const(&inner_expr)
+            } else {
+                false
+            }
+        }
+        // current_timestamp is the same as calling now()
+        ast::Expr::NameRef(name_ref) => {
+            if let Some(child) = name_ref.syntax().first_child_or_token() {
+                if child.kind() == SyntaxKind::CURRENT_TIMESTAMP_KW {
+                    return true;
+                }
+            }
+            false
+        }
         _ => false,
     }
 }
@@ -55,8 +74,7 @@ fn is_non_volatile(expr: &ast::Expr) -> bool {
 const NON_VOLATILE_BUILT_IN_FUNCTIONS: &str = include_str!("non_volatile_built_in_functions.txt");
 
 pub(crate) fn adding_field_with_default(ctx: &mut Linter, parse: &Parse<SourceFile>) {
-    let message =
-        "Adding a generated column requires a table rewrite with an `ACCESS EXCLUSIVE` lock.";
+    let message = "Adding a generated column requires a table rewrite with an `ACCESS EXCLUSIVE` lock. In Postgres versions 11+, non-VOLATILE DEFAULTs can be added without a rewrite.";
     let help = "Add the column as nullable, backfill existing rows, and add a trigger to update the column on write instead.";
     let file = parse.tree();
     // TODO: use match_ast! like in #api_walkthrough
@@ -70,23 +88,29 @@ pub(crate) fn adding_field_with_default(ctx: &mut Linter, parse: &Parse<SourceFi
                                 let Some(expr) = default.expr() else {
                                     continue;
                                 };
-                                if is_const_expr(&expr) || is_non_volatile(&expr) {
+                                if ctx.settings.pg_version > Version::new(11, None, None)
+                                    && is_non_volatile_or_const(&expr)
+                                {
                                     continue;
                                 }
-                                ctx.report(Violation::new(
-                                    Rule::AddingFieldWithDefault,
-                                    message.into(),
-                                    expr.syntax().text_range(),
-                                    help.to_string(),
-                                ))
+                                ctx.report(
+                                    Violation::for_node(
+                                        Rule::AddingFieldWithDefault,
+                                        message.into(),
+                                        expr.syntax(),
+                                    )
+                                    .help(help),
+                                )
                             }
                             ast::Constraint::GeneratedConstraint(generated) => {
-                                ctx.report(Violation::new(
-                                    Rule::AddingFieldWithDefault,
-                                    message.into(),
-                                    generated.syntax().text_range(),
-                                    help.to_string(),
-                                ));
+                                ctx.report(
+                                    Violation::for_node(
+                                        Rule::AddingFieldWithDefault,
+                                        message.into(),
+                                        generated.syntax(),
+                                    )
+                                    .help(help),
+                                );
                             }
                             _ => (),
                         }
@@ -102,13 +126,10 @@ mod test {
     use insta::assert_debug_snapshot;
 
     use crate::Rule;
-    use crate::test_utils::lint;
+    use crate::test_utils::{lint, lint_with_postgres_version};
 
     #[test]
     fn docs_example_ok_post_pg_11() {
-        // TODO: differing from squawk because we aren't checking the postgres
-        // version, maybe we should be default to a more recent version like 15
-        // instead of 11?
         let sql = r#"
 -- instead of
 ALTER TABLE "core_recipe" ADD COLUMN "foo" integer DEFAULT 10;
@@ -180,6 +201,33 @@ ALTER TABLE "core_recipe" ADD COLUMN "foo" boolean DEFAULT true;
         assert_debug_snapshot!(errors);
     }
 
+    #[test]
+    fn default_empty_array_ok() {
+        let sql = r#"
+alter table t add column a double precision[] default array[]::double precision[];
+
+alter table t add column b bigint[] default cast(array[] as bigint[]);
+
+alter table t add column c text[] default array['foo', 'bar']::text[];
+            "#;
+
+        let errors = lint(sql, Rule::AddingFieldWithDefault);
+        assert!(errors.is_empty());
+        assert_debug_snapshot!(errors);
+    }
+
+    #[test]
+    fn default_with_const_bin_expr() {
+        let sql = r#"
+ALTER TABLE assessments
+ADD COLUMN statistics_last_updated_at timestamptz NOT NULL DEFAULT now() - interval '100 years';
+            "#;
+
+        let errors = lint(sql, Rule::AddingFieldWithDefault);
+        assert!(errors.is_empty());
+        assert_debug_snapshot!(errors);
+    }
+
     #[test]
     fn default_str_ok() {
         let sql = r#"
@@ -239,6 +287,7 @@ ALTER TABLE "core_recipe" ADD COLUMN "foo" timestamptz DEFAULT now(123);
         assert!(!errors.is_empty());
         assert_debug_snapshot!(errors);
     }
+
     #[test]
     fn default_func_now_ok() {
         let sql = r#"
@@ -251,14 +300,25 @@ ALTER TABLE "core_recipe" ADD COLUMN "foo" timestamptz DEFAULT now();
         assert_debug_snapshot!(errors);
     }
 
+    #[test]
+    fn default_func_current_timestamp_ok() {
+        let sql = r#"
+alter table t add column c timestamptz default current_timestamp;
+        "#;
+
+        let errors = lint(sql, Rule::AddingFieldWithDefault);
+        assert!(errors.is_empty());
+        assert_debug_snapshot!(errors);
+    }
+
     #[test]
     fn add_numbers_ok() {
-        // This should be okay, but we don't handle expressions like this at the moment.
         let sql = r#"
 alter table account_metadata add column blah integer default 2 + 2;
         "#;
 
         let errors = lint(sql, Rule::AddingFieldWithDefault);
+        assert!(errors.is_empty());
         assert_debug_snapshot!(errors);
     }
 
@@ -273,4 +333,16 @@ ADD COLUMN bar numeric GENERATED ALWAYS AS (bar + baz) STORED;
         assert!(!errors.is_empty());
         assert_debug_snapshot!(errors);
     }
+
+    #[test]
+    fn docs_example_error_on_pg_11() {
+        let sql = r#"
+-- instead of
+ALTER TABLE "core_recipe" ADD COLUMN "foo" integer DEFAULT 10;
+        "#;
+
+        let errors = lint_with_postgres_version(sql, Rule::AddingFieldWithDefault, "11");
+        assert!(!errors.is_empty());
+        assert_debug_snapshot!(errors);
+    }
 }
