@@ -3,9 +3,10 @@ use std::collections::HashMap;
 use squawk_syntax::{
     Parse, SourceFile,
     ast::{self, AstNode},
+    identifier::Identifier,
 };
 
-use crate::{Edit, Fix, Linter, Rule, Violation, identifier::Identifier};
+use crate::{Edit, Fix, Linter, Rule, Violation};
 
 #[derive(PartialEq)]
 enum Constraint {
@@ -58,15 +59,11 @@ pub(crate) fn prefer_robust_stmts(ctx: &mut Linter, parse: &Parse<SourceFile>) {
                                 continue;
                             }
 
-                            let fix = if let Some(constraint_token) =
-                                drop_constraint.constraint_token()
-                            {
+                            let fix = drop_constraint.constraint_token().map(|constraint_token| {
                                 let at = constraint_token.text_range().end();
                                 let edit = Edit::insert(" if exists", at);
-                                Some(Fix::new("Insert `if exists`", vec![edit]))
-                            } else {
-                                None
-                            };
+                                Fix::new("Insert `if exists`", vec![edit])
+                            });
 
                             (ActionErrorMessage::IfExists, fix)
                         }
@@ -75,13 +72,11 @@ pub(crate) fn prefer_robust_stmts(ctx: &mut Linter, parse: &Parse<SourceFile>) {
                                 continue;
                             }
 
-                            let fix = if let Some(column_token) = add_column.column_token() {
+                            let fix = add_column.column_token().map(|column_token| {
                                 let at = column_token.text_range().end();
                                 let edit = Edit::insert(" if not exists", at);
-                                Some(Fix::new("Insert `if not exists`", vec![edit]))
-                            } else {
-                                None
-                            };
+                                Fix::new("Insert `if not exists`", vec![edit])
+                            });
                             (ActionErrorMessage::IfNotExists, fix)
                         }
                         ast::AlterTableAction::ValidateConstraint(validate_constraint) => {
@@ -113,13 +108,11 @@ pub(crate) fn prefer_robust_stmts(ctx: &mut Linter, parse: &Parse<SourceFile>) {
                                 continue;
                             }
 
-                            let fix = if let Some(column_token) = drop_column.column_token() {
+                            let fix = drop_column.column_token().map(|column_token| {
                                 let at = column_token.text_range().end();
                                 let edit = Edit::insert(" if exists", at);
-                                Some(Fix::new("Insert `if exists`", vec![edit]))
-                            } else {
-                                None
-                            };
+                                Fix::new("Insert `if exists`", vec![edit])
+                            });
                             (ActionErrorMessage::IfExists, fix)
                         }
                         _ => (ActionErrorMessage::None, None),
@@ -142,13 +135,8 @@ pub(crate) fn prefer_robust_stmts(ctx: &mut Linter, parse: &Parse<SourceFile>) {
                     };
 
                     ctx.report(
-                        Violation::new(
-                            Rule::PreferRobustStmts,
-                            message,
-                            action.syntax().text_range(),
-                            None,
-                        )
-                        .with_fix(fix),
+                        Violation::for_node(Rule::PreferRobustStmts, message, action.syntax())
+                            .fix(fix),
                     );
                 }
             }
@@ -157,90 +145,75 @@ pub(crate) fn prefer_robust_stmts(ctx: &mut Linter, parse: &Parse<SourceFile>) {
                     && create_index.name().is_some()
                     && (create_index.concurrently_token().is_some() || !inside_transaction) =>
             {
-                let fix = if let Some(name) = create_index.name() {
+                let fix = create_index.name().map(|name| {
                     let at = name.syntax().text_range().start();
                     let edit = Edit::insert("if not exists ", at);
-                    Some(Fix::new("Insert `if not exists`", vec![edit]))
-                } else {
-                    None
-                };
-                ctx.report(Violation::new(
+                    Fix::new("Insert `if not exists`", vec![edit])
+                });
+                ctx.report(Violation::for_node(
                     Rule::PreferRobustStmts,
                     "Missing `IF NOT EXISTS`, the migration can't be rerun if it fails part way through.".into(),
-                    create_index.syntax().text_range(),
-                    "Use an explicit name for a concurrently created index".to_string(),
-                ).with_fix(fix));
+                    create_index.syntax(),
+                ).help("Use an explicit name for a concurrently created index").fix(fix));
             }
             ast::Stmt::CreateTable(create_table)
                 if create_table.if_not_exists().is_none() && !inside_transaction =>
             {
-                let fix = if let Some(table_token) = create_table.table_token() {
+                let fix = create_table.table_token().map(|table_token| {
                     let at = table_token.text_range().end();
                     let edit = Edit::insert(" if not exists", at);
-                    Some(Fix::new("Insert `if not exists`", vec![edit]))
-                } else {
-                    None
-                };
+                    Fix::new("Insert `if not exists`", vec![edit])
+                });
 
-                ctx.report(Violation::new(
+                ctx.report(Violation::for_node(
                     Rule::PreferRobustStmts,
                     "Missing `IF NOT EXISTS`, the migration can't be rerun if it fails part way through.".into(),
-                    create_table.syntax().text_range(),
-                    None,
-                ).with_fix(fix));
+                    create_table.syntax(),
+                ).fix(fix));
             }
             ast::Stmt::DropIndex(drop_index)
                 if drop_index.if_exists().is_none() && !inside_transaction =>
             {
-                let fix = if let Some(first_index) = drop_index.paths().next() {
+                let fix = drop_index.paths().next().map(|first_index| {
                     let at = first_index.syntax().text_range().start();
                     let edit = Edit::insert("if exists ", at);
-                    Some(Fix::new("Insert `if exists`", vec![edit]))
-                } else {
-                    None
-                };
+                    Fix::new("Insert `if exists`", vec![edit])
+                });
 
-                ctx.report(Violation::new(
+                ctx.report(Violation::for_node(
                     Rule::PreferRobustStmts,
                     "Missing `IF EXISTS`, the migration can't be rerun if it fails part way through.".into(),
-                    drop_index.syntax().text_range(),
-                    None,
-                ).with_fix(fix));
+                    drop_index.syntax(),
+                ).fix(fix));
             }
             ast::Stmt::DropTable(drop_table)
                 if drop_table.if_exists().is_none() && !inside_transaction =>
             {
-                let fix = if let Some(table_token) = drop_table.table_token() {
+                let fix = drop_table.table_token().map(|table_token| {
                     let at = table_token.text_range().end();
                     let edit = Edit::insert(" if exists", at);
-                    Some(Fix::new("Insert `if exists`", vec![edit]))
-                } else {
-                    None
-                };
-                ctx.report(Violation::new(
+                    Fix::new("Insert `if exists`", vec![edit])
+                });
+                ctx.report(Violation::for_node(
                     Rule::PreferRobustStmts,
                     "Missing `IF EXISTS`, the migration can't be rerun if it fails part way through.".into(),
-                    drop_table.syntax().text_range(),
-                    None,
-                ).with_fix(fix));
+                    drop_table.syntax(),
+                ).fix(fix));
             }
             ast::Stmt::DropType(drop_type)
                 if drop_type.if_exists().is_none() && !inside_transaction =>
             {
-                let fix = if let Some(type_token) = drop_type.type_token() {
+                let fix = drop_type.type_token().map(|type_token| {
                     let at = type_token.text_range().end();
                     let edit = Edit::insert(" if exists", at);
-                    Some(Fix::new("Insert `if exists`", vec![edit]))
-                } else {
-                    None
-                };
+                    Fix::new("Insert `if exists`", vec![edit])
+                });
 
-                ctx.report(Violation::new(
+                ctx.report(Violation::for_node(
                     Rule::PreferRobustStmts,
                     "Missing `IF EXISTS`, the migration can't be rerun if it fails part way through.".into(),
-                    drop_type.syntax().text_range(),
-                    None,
-                ).with_fix(fix));
+                    drop_type.syntax(),
+                ).fix(fix));
             }
             _ => (),
         }
@@ -251,46 +224,13 @@ pub(crate) fn prefer_robust_stmts(ctx: &mut Linter, parse: &Parse<SourceFile>) {
 mod test {
     use insta::{assert_debug_snapshot, assert_snapshot};
 
-    use crate::{Edit, Linter, Rule, test_utils::{lint, lint_with_assume_in_transaction}};
+    use crate::{
+        Rule,
+        test_utils::{fix_sql, lint, lint_with_assume_in_transaction},
+    };
 
     fn fix(sql: &str) -> String {
-        let file = squawk_syntax::SourceFile::parse(sql);
-        assert_eq!(file.errors().len(), 0);
-        assert_eq!(file.errors().len(), 0, "Shouldn't start with syntax errors");
-        let mut linter = Linter::from([Rule::PreferRobustStmts]);
-        let errors = linter.lint(file, sql);
-        assert!(!errors.is_empty(), "Should start with linter errors");
-
-        let fixes = errors.into_iter().flat_map(|x| x.fix).collect::<Vec<_>>();
-
-        let mut result = sql.to_string();
-
-        let mut all_edits: Vec<&Edit> = fixes.iter().flat_map(|fix| &fix.edits).collect();
-
-        all_edits.sort_by(|a, b| b.text_range.start().cmp(&a.text_range.start()));
-
-        for edit in all_edits {
-            let start: usize = edit.text_range.start().into();
-            let end: usize = edit.text_range.end().into();
-            let text = edit.text.as_ref().map_or("", |v| v);
-            result.replace_range(start..end, text);
-        }
-
-        let file = squawk_syntax::SourceFile::parse(&result);
-        assert_eq!(
-            file.errors().len(),
-            0,
-            "Shouldn't introduce any syntax errors"
-        );
-        let mut linter = Linter::from([Rule::PreferRobustStmts]);
-        let errors = linter.lint(file, &result);
-        assert_eq!(
-            errors.len(),
-            0,
-            "Fixes should remove all the linter errors."
-        );
-
-        result
+        fix_sql(sql, Rule::PreferRobustStmts)
     }
 
     #[test]
