@@ -11,11 +11,10 @@ pub(crate) fn renaming_table(ctx: &mut Linter, parse: &Parse<SourceFile>) {
         if let ast::Stmt::AlterTable(alter_table) = stmt {
             for action in alter_table.actions() {
                 if let ast::AlterTableAction::RenameTable(rename_table) = action {
-                    ctx.report(Violation::new(
+                    ctx.report(Violation::for_node(
                         Rule::RenamingTable,
                         "Renaming a table may break existing clients.".into(),
-                        rename_table.syntax().text_range(),
-                        None,
+                        rename_table.syntax(),
                     ));
                 }
             }
