@@ -1,25 +1,28 @@
 use anyhow::{Context, Result};
-use line_index::LineIndex;
 use log::info;
 use lsp_server::{Connection, Message, Notification, Response};
 use lsp_types::{
     CodeAction, CodeActionKind, CodeActionOptions, CodeActionOrCommand, CodeActionParams,
-    CodeActionProviderCapability, CodeActionResponse, CodeDescription, Diagnostic,
-    DiagnosticSeverity, DidChangeTextDocumentParams, DidCloseTextDocumentParams,
-    DidOpenTextDocumentParams, GotoDefinitionParams, GotoDefinitionResponse, InitializeParams,
-    Location, Position, PublishDiagnosticsParams, Range, ServerCapabilities,
-    TextDocumentSyncCapability, TextDocumentSyncKind, TextEdit, Url, WorkDoneProgressOptions,
-    WorkspaceEdit,
+    CodeActionProviderCapability, CodeActionResponse, Command, Diagnostic,
+    DidChangeTextDocumentParams, DidCloseTextDocumentParams, DidOpenTextDocumentParams,
+    GotoDefinitionParams, GotoDefinitionResponse, InitializeParams, Location, Position,
+    PublishDiagnosticsParams, Range, ServerCapabilities, TextDocumentSyncCapability,
+    TextDocumentSyncKind, Url, WorkDoneProgressOptions, WorkspaceEdit,
     notification::{
         DidChangeTextDocument, DidCloseTextDocument, DidOpenTextDocument, Notification as _,
         PublishDiagnostics,
     },
     request::{CodeActionRequest, GotoDefinition, Request},
 };
-use serde::{Deserialize, Serialize};
-use squawk_linter::Linter;
 use squawk_syntax::{Parse, SourceFile};
 use std::collections::HashMap;
+
+use diagnostic::DIAGNOSTIC_NAME;
+
+use crate::diagnostic::AssociatedDiagnosticData;
+mod diagnostic;
+mod ignore;
+mod lint;
 mod lsp_utils;
 
 struct DocumentState {
@@ -152,11 +155,86 @@ fn handle_code_action(
 
     let mut actions = Vec::new();
 
-    for mut diagnostic in params.context.diagnostics {
-        if let Some(data) = diagnostic.data.take() {
-            let associated_data: AssociatedDiagnosticData =
-                serde_json::from_value(data).context("deserializing diagnostic data")?;
+    for mut diagnostic in params
+        .context
+        .diagnostics
+        .into_iter()
+        .filter(|diagnostic| diagnostic.source.as_deref() == Some(DIAGNOSTIC_NAME))
+    {
+        let Some(rule_name) = diagnostic.code.as_ref().map(|x| match x {
+            lsp_types::NumberOrString::String(s) => s.clone(),
+            lsp_types::NumberOrString::Number(n) => n.to_string(),
+        }) else {
+            continue;
+        };
+        let Some(data) = diagnostic.data.take() else {
+            continue;
+        };
 
+        let associated_data: AssociatedDiagnosticData =
+            serde_json::from_value(data).context("deserializing diagnostic data")?;
+
+        if let Some(ignore_line_edit) = associated_data.ignore_line_edit {
+            let disable_line_action = CodeAction {
+                title: format!("Disable {rule_name} for this line"),
+                kind: Some(CodeActionKind::QUICKFIX),
+                diagnostics: Some(vec![diagnostic.clone()]),
+                edit: Some(WorkspaceEdit {
+                    changes: Some({
+                        let mut changes = HashMap::new();
+                        changes.insert(uri.clone(), vec![ignore_line_edit]);
+                        changes
+                    }),
+                    ..Default::default()
+                }),
+                command: None,
+                is_preferred: Some(false),
+                disabled: None,
+                data: None,
+            };
+            actions.push(CodeActionOrCommand::CodeAction(disable_line_action));
+        }
+        if let Some(ignore_file_edit) = associated_data.ignore_file_edit {
+            let disable_file_action = CodeAction {
+                title: format!("Disable {rule_name} for the entire file"),
+                kind: Some(CodeActionKind::QUICKFIX),
+                diagnostics: Some(vec![diagnostic.clone()]),
+                edit: Some(WorkspaceEdit {
+                    changes: Some({
+                        let mut changes = HashMap::new();
+                        changes.insert(uri.clone(), vec![ignore_file_edit]);
+                        changes
+                    }),
+                    ..Default::default()
+                }),
+                command: None,
+                is_preferred: Some(false),
+                disabled: None,
+                data: None,
+            };
+            actions.push(CodeActionOrCommand::CodeAction(disable_file_action));
+        }
+
+        let title = format!("Show documentation for {rule_name}");
+        let documentation_action = CodeAction {
+            title: title.clone(),
+            kind: Some(CodeActionKind::QUICKFIX),
+            diagnostics: Some(vec![diagnostic.clone()]),
+            edit: None,
+            command: Some(Command {
+                title,
+                command: "vscode.open".to_string(),
+                arguments: Some(vec![serde_json::to_value(format!(
+                    "https://squawkhq.com/docs/{rule_name}"
+                ))?]),
+            }),
+            is_preferred: Some(false),
+            disabled: None,
+            data: None,
+        };
+        actions.push(CodeActionOrCommand::CodeAction(documentation_action));
+
+        if !associated_data.title.is_empty() && !associated_data.edits.is_empty() {
             let fix_action = CodeAction {
                 title: associated_data.title,
                 kind: Some(CodeActionKind::QUICKFIX),
@@ -174,9 +252,8 @@ fn handle_code_action(
                 disabled: None,
                 data: None,
             };
-
             actions.push(CodeActionOrCommand::CodeAction(fix_action));
-        };
+        }
     }
 
     let result: CodeActionResponse = actions;
@@ -228,7 +305,7 @@ fn handle_did_open(
     let content = documents.get(&uri).map_or("", |doc| &doc.content);
 
     // TODO: we need a better setup for "run func when input changed"
-    let diagnostics = lint(content);
+    let diagnostics = lint::lint(content);
     publish_diagnostics(connection, uri, version, diagnostics)?;
 
     Ok(())
@@ -251,7 +328,7 @@ fn handle_did_change(
         lsp_utils::apply_incremental_changes(&doc_state.content, params.content_changes);
     doc_state.version = version;
 
-    let diagnostics = lint(&doc_state.content);
+    let diagnostics = lint::lint(&doc_state.content);
     publish_diagnostics(connection, uri, version, diagnostics)?;
 
     Ok(())
@@ -285,109 +362,6 @@ fn handle_did_close(
     Ok(())
 }
 
-// Based on Ruff's setup for LSP diagnostic edits
-// see: https://github.com/astral-sh/ruff/blob/1a368b0bf97c3d0246390679166bbd2d589acf39/crates/ruff_server/src/lint.rs#L31
-/// This is serialized on the diagnostic `data` field.
-#[derive(Serialize, Deserialize, Debug, Clone)]
-struct AssociatedDiagnosticData {
-    /// The message describing what the fix does, if it exists, or the diagnostic name otherwise.
-    title: String,
-    /// Edits to fix the diagnostic. If this is empty, a fix
-    /// does not exist.
-    edits: Vec<lsp_types::TextEdit>,
-}
-
-fn lint(content: &str) -> Vec<Diagnostic> {
-    let parse: Parse<SourceFile> = SourceFile::parse(content);
-    let parse_errors = parse.errors();
-    let mut linter = Linter::with_all_rules();
-    let violations = linter.lint(parse, content);
-    let line_index = LineIndex::new(content);
-
-    let mut diagnostics = Vec::with_capacity(violations.len() + parse_errors.len());
-
-    for error in parse_errors {
-        let range_start = error.range().start();
-        let range_end = error.range().end();
-        let start_line_col = line_index.line_col(range_start);
-        let mut end_line_col = line_index.line_col(range_end);
-
-        if start_line_col.line == end_line_col.line && start_line_col.col == end_line_col.col {
-            end_line_col.col += 1;
-        }
-
-        let diagnostic = Diagnostic {
-            range: Range::new(
-                Position::new(start_line_col.line, start_line_col.col),
-                Position::new(end_line_col.line, end_line_col.col),
-            ),
-            severity: Some(DiagnosticSeverity::ERROR),
-            code: Some(lsp_types::NumberOrString::String(
-                "syntax-error".to_string(),
-            )),
-            code_description: Some(CodeDescription {
-                href: Url::parse("https://squawkhq.com/docs/syntax-error").unwrap(),
-            }),
-            source: Some("squawk".to_string()),
-            message: error.message().to_string(),
-            ..Default::default()
-        };
-        diagnostics.push(diagnostic);
-    }
-
-    for violation in violations {
-        let range_start = violation.text_range.start();
-        let range_end = violation.text_range.end();
-        let start_line_col = line_index.line_col(range_start);
-        let mut end_line_col = line_index.line_col(range_end);
-
-        if start_line_col.line == end_line_col.line && start_line_col.col == end_line_col.col {
-            end_line_col.col += 1;
-        }
-
-        let data = if let Some(fix) = violation.fix {
-            Some(AssociatedDiagnosticData {
-                title: fix.title,
-                edits: fix
-                    .edits
-                    .into_iter()
-                    .filter_map(|x| {
-                        let start_line = line_index.try_line_col(x.text_range.start())?;
-                        let end_line = line_index.try_line_col(x.text_range.end())?;
-                        let range = Range::new(
-                            Position::new(start_line.line, start_line.col),
-                            Position::new(end_line.line, end_line.col),
-                        );
-                        Some(TextEdit::new(range, x.text.unwrap_or_default()))
-                    })
-                    .collect(),
-            })
-        } else {
-            None
-        };
-
-        let diagnostic = Diagnostic {
-            range: Range::new(
-                Position::new(start_line_col.line, start_line_col.col),
-                Position::new(end_line_col.line, end_line_col.col),
-            ),
-            severity: Some(DiagnosticSeverity::WARNING),
-            code: Some(lsp_types::NumberOrString::String(
-                violation.code.to_string(),
-            )),
-            code_description: Some(CodeDescription {
-                href: Url::parse(&format!("https://squawkhq.com/docs/{}", violation.code)).unwrap(),
-            }),
-            source: Some("squawk".to_string()),
-            message: violation.message,
-            data: data.map(|d| serde_json::to_value(d).unwrap()),
-            ..Default::default()
-        };
-        diagnostics.push(diagnostic);
-    }
-    diagnostics
-}
-
 #[derive(serde::Deserialize)]
 struct SyntaxTreeParams {
     #[serde(rename = "textDocument")]
